@@ -9,28 +9,44 @@ from trytond.transaction import Transaction
 class Invoice(metaclass=PoolMeta):
     __name__ = 'account.invoice'
 
+    shipments = fields.Function(fields.Char("Shipments"), 'get_shipments')
+
+    def get_shipments(self, name):
+        shipments = {
+            m.shipment.rec_name for l in self.lines
+            for m in l.stock_moves if m.shipment}
+        return ', '.join(sorted(shipments))
+
     @classmethod
     def _post(cls, invoices):
         pool = Pool()
         Move = pool.get('stock.move')
+        transaction = Transaction()
+        context = transaction.context
         super()._post(invoices)
         moves = sum((l.stock_moves for i in invoices for l in i.lines), ())
         if moves:
-            Move.__queue__.update_unit_price(moves)
+            with transaction.set_context(
+                    queue_batch=context.get('queue_batch', True)):
+                Move.__queue__.update_unit_price(moves)
 
 
 class InvoiceLineStockMove(ModelSQL):
     'Invoice Line - Stock Move'
     __name__ = 'account.invoice.line-stock.move'
 
-    invoice_line = fields.Many2One('account.invoice.line', 'Invoice Line',
-        required=True, select=True, ondelete='CASCADE')
-    stock_move = fields.Many2One('stock.move', 'Stock Move', required=True,
-        select=True, ondelete='CASCADE')
+    invoice_line = fields.Many2One(
+        'account.invoice.line', "Invoice Line",
+        required=True, ondelete='CASCADE')
+    stock_move = fields.Many2One(
+        'stock.move', "Stock Move", required=True, ondelete='CASCADE')
 
 
 class InvoiceLine(metaclass=PoolMeta):
     __name__ = 'account.invoice.line'
+
+    warehouse = fields.Function(fields.Many2One(
+            'stock.location', "Warehouse"), 'get_warehouse')
     stock_moves = fields.Many2Many(
         'account.invoice.line-stock.move', 'invoice_line', 'stock_move',
         "Stock Moves",
@@ -52,15 +68,14 @@ class InvoiceLine(metaclass=PoolMeta):
             'invisible': (
                 (Eval('type') != 'line')
                 | ~Eval('product')),
-            },
-        depends=['type', 'product_uom_category', 'invoice', 'invoice_type'])
+            })
+    shipments = fields.Function(fields.Char("Shipments"), 'get_shipments')
     correction = fields.Boolean(
         "Correction",
         states={
             'invisible': ((Eval('_parent_invoice', {}).get('type') == 'out')
                 | (Eval('invoice_type') == 'out')),
             },
-        depends=['invoice_type'],
         help="Check to correct price of already posted invoice.")
 
     @classmethod
@@ -72,6 +87,17 @@ class InvoiceLine(metaclass=PoolMeta):
     def default_correction(cls):
         return False
 
+    def get_warehouse(self, name):
+        if (self.invoice_type == 'out'
+                or (self.invoice and self.invoice.type) == 'out'):
+            warehouses = set(filter(None, [
+                        m.from_location.warehouse for m in self.stock_moves]))
+        else:
+            warehouses = set(filter(None, [
+                        m.to_location.warehouse for m in self.stock_moves]))
+        if warehouses:
+            return list(warehouses)[0].id
+
     @property
     def moved_quantity(self):
         'The quantity from linked stock moves in line unit'
@@ -81,18 +107,27 @@ class InvoiceLine(metaclass=PoolMeta):
         for stock_move in self.stock_moves:
             if stock_move.state != 'cancelled':
                 quantity += Uom.compute_qty(
-                    stock_move.uom, stock_move.quantity, self.unit)
+                    stock_move.unit, stock_move.quantity, self.unit)
         return quantity
 
+    def get_shipments(self, name):
+        shipments = {
+            m.shipment.rec_name for m in self.stock_moves if m.shipment}
+        return ', '.join(sorted(shipments))
+
     @classmethod
     def write(cls, *args):
         pool = Pool()
         Move = pool.get('stock.move')
+        transaction = Transaction()
+        context = transaction.context
         super().write(*args)
         lines = sum(args[0:None:2], [])
         moves = sum((l.stock_moves for l in lines), ())
         if moves:
-            Move.__queue__.update_unit_price(moves)
+            with transaction.set_context(
+                    queue_batch=context.get('queue_batch', True)):
+                Move.__queue__.update_unit_price(moves)
 
     @classmethod
     def copy(cls, lines, default=None):
